const http = require('http');
const { Semantics, Semantic } = require('./src/semantics');
const { Generators, Generator } = require('./src/generators');

const process = (config, key) =>

  new Promise( (resolve, reject) => {
    var data = Object.assign( { key, version: '3' }, config);
    if (!config.objects) {
      config.objects = {}
    }
    const generators = new Generators(data.generators.map( (g) => new Generator(g[0], g[1]) ) )
    delete data['generators']
    const semantics = new Semantics(data.semantics.map( (g) => new Semantic(g[0], g[1]) ) )
    data = JSON.stringify(data);

    const options = {
      hostname: '184.67.27.82',
      port: 5000,
      path: '/process',
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'Content-Length': data.length
      }
    }

    var response = ''
    const req = http.request(options, (res) => {
      res.on('data', (d) => {
        if (res.statusCode == 200) {
          response += d.toString()
        } else {
          reject(d.toString())
        }
      })
      res.on('end', (d) => {
        response = JSON.parse(response)
        if (response.results) {
          response.results = response.results.map( (result) => semantics.apply(config.objects, result) )
          response.generated = response.results.map( (result) => generators.apply(result) )
        }
        resolve(response);
      })
    })

    req.on('error', (error) => {
        console.log('in ERROR');
      const message = `Error from server ${error}.`;
      reject(message)
    })

    req.write(data)
    req.end()
  });

module.exports = { process }
