Generator = require('./generators.js').Generator;
Semantic = require('./semantics.js').Semantic;
module.exports = {
  contexts: [{"marker": "earn", "units": "dollars", "amount": 10, "who": "joe", "period": "week"}, {"marker": "earn", "units": "dollars", "amount": 25, "who": "sally", "period": "week"}, {"marker": "worked", "who": "sally", "duration": 10, "units": "weekConcept"}, {"marker": "worked", "who": "joe", "duration": 15, "units": "weekConcept"}, {"isQuery": true, "marker": "earn", "units": "dollars", "amount": null, "who": "joe", "period": null}, {"isQuery": true, "marker": "earn", "units": "dollars", "amount": null, "who": "sally", "period": null}],
  expected: ["joe earns 10 dollars per week", "sally earns 25 dollars per week", "sally worked 10 weeks", "joe worked 15 weeks", "joe earned 150 dollars", "sally earned 250 dollars"],
  expected_semantics: [{"marker": "earn", "units": "dollars", "amount": 10, "who": "joe", "period": "week"}, {"marker": "earn", "units": "dollars", "amount": 25, "who": "sally", "period": "week"}, {"marker": "worked", "who": "sally", "duration": 10, "units": "weekConcept"}, {"marker": "worked", "who": "joe", "duration": 15, "units": "weekConcept"}, {"isQuery": true, "marker": "response", "units": "dollars", "amount": null, "who": "joe", "period": null, "earnings": 150}, {"isQuery": true, "marker": "response", "units": "dollars", "amount": null, "who": "sally", "period": null, "earnings": 250}],
  generators: [
    new Generator( (context) => context.marker == 'earn', (g, context) => `${g(context.who)} earns ${g(context.amount)} ${g(context.units)} per ${context.period}` ),
    new Generator( (context) => context.marker == 'weekConcept' && context.duration == 1, (g, context) => `${context.duration} week` ),
    new Generator( (context) => context.marker == 'weekConcept' && context.duration > 1, (g, context) => `${context.duration} weeks` ),
    new Generator( (context) => context.marker == 'worked', (g, context) => `${g(context.who)} worked ${ g({ marker: context.units, duration: context.duration}) }` ),
    new Generator( (context) => context.marker == 'response', (g, context) => `${context.who} earned ${context.earnings} ${context.units}` ),
  ],
  semantics: [
    new Semantic( (global, context) => context.marker == 'earn' && context.isQuery, (global, context) => { 
          context.marker = 'response'; 
          var employee_record = global.employees.find( (er) => er.name == context.who )
          let totalIncome = 0
          global.workingTime.forEach( (wt) => {
            if (wt.name == context.who) {
              totalIncome += employee_record.earnings_per_period * wt.number_of_time_units
            }
          });
          context.earnings = totalIncome
         } ),
    new Semantic( (global, context) => context.marker == 'earn', (global, context) => { 
          if (! global.employees ) {
            global.employees = []
          }
          global.employees.push({ name: context.who, earnings_per_period: context.amount, period: context.period, units: 'dollars' })
         } ),
    new Semantic( (global, context) => context.marker == 'worked', (global, context) => { 
          if (! global.workingTime ) {
            global.workingTime = []
          }
          global.workingTime.push({ name: context.who, number_of_time_units: context.duration, time_units: context.units })
         } ),
  ]
};