class Semantic {
  constructor(matcher, action) {
    this.matcher = matcher;
    this.action = action;
  }

  toString() {
    return `Semantic(${this.matcher}, ${this.action})`;
  }

  matches(global, context) {
    return this.matcher(global, context)
  }

  apply(global, context) {
    const context_prime = Object.assign({}, context);
    this.action(global, context_prime);
    return context_prime;
  }
}

class Semantics {
  constructor(semantics) {
    this.semantics = semantics
    this.logs = []
  };

  apply(global, contexts) {
    const contexts_prime = [];
    contexts.forEach( (context) => {
      let context_prime = context;
      for (const isemantic in this.semantics) {
        const semantic = this.semantics[isemantic]
        if (semantic.matches(global, context)) {
          context_prime = semantic.apply(global, context);
          this.logs.push(`Semantics: applied ${semantic.toString()}\n to\n ${JSON.stringify(context)}`)
          break
        }
      }
      contexts_prime.push(context_prime);
    });
    return contexts_prime;
  }
}

module.exports = { Semantic, Semantics };
